#!/usr/bin/env python3
"""
The evaluation software for UCCA layer 1.
"""

from optparse import OptionParser

from scripts import ucca_db
from ucca import convert
from ucca.evaluation import evaluate


##############################################################################
# Returns the command line parser.
##############################################################################
def cmd_line_parser():
    usage = "usage: %prog [options]\n"
    opt_parser = OptionParser(usage=usage)
    opt_parser.add_option("--db", "-d", dest="db_filename",
                          action="store", type="string",
                          help="the db file name")
    opt_parser.add_option("--host", "--hst", dest="host",
                          action="store", type="string",
                          help="the host name")
    opt_parser.add_option("--pid", "-p", dest="pid", action="store",
                          type="int", help="the passage ID")
    opt_parser.add_option("--from_xids", "-x", dest="from_xids",
                          action="store_true", help="interpret the ref \
                          and the guessed parameters as Xids in the db")
    opt_parser.add_option("--guessed", "-g", dest="guessed", action="store",
                          type="string", help="if a db is defined - \
                          the username for the guessed annotation; \
                          else - the xml file name for the guessed annotation")
    opt_parser.add_option("--ref", "-r", dest="ref", action="store",
                          type="string", help="if a db is defined - \
                          the username for the reference annotation; else - \
                          the xml file name for the reference annotation")
    opt_parser.add_option("--units", "-u", dest="units", action="store_true",
                          help="the units the annotations have in common, \
                          and those each has separately")
    opt_parser.add_option("--fscore", "-f", dest="fscore", action="store_true",
                          help="outputs the traditional P,R,F \
                          instead of the scene structure evaluation")
    opt_parser.add_option("--debug", dest="debug", action="store_true",
                          help="run in debug mode")
    opt_parser.add_option("--errors", "-e", dest="errors", action="store_true",
                          help="prints the error distribution\
                          according to its frequency")
    return opt_parser


def main():
    opt_parser = cmd_line_parser()
    (options, args) = opt_parser.parse_args()
    if len(args) > 0:
        opt_parser.error("all arguments must be flagged")

    if (options.guessed is None) or (options.ref is None) or (options.db_filename is None):
        opt_parser.error("missing arguments. type --help for help.")
    if options.pid is not None and options.from_xids is not None:
        opt_parser.error("inconsistent parameters. \
        you can't have both a pid and from_xids paramters.")

    keys = [options.guessed, options.ref]
    if options.from_xids:
        xmls = ucca_db.get_by_xids(options.db_filename, options.host, keys)
    else:
        xmls = ucca_db.get_xml_trees(options.db_filename, options.host,
                                     options.pid, keys)

    guessed, ref = [convert.from_site(x) for x in xmls]
    if options.units or options.fscore or options.errors:
        evaluate(guessed, ref,
                 units=options.units, fscore=options.fscore, errors=options.errors, verbose=True)


if __name__ == '__main__':
    main()



from argparse import ArgumentParser

from tqdm import tqdm

from ucca import constructions
from ucca.ioutil import get_passages_with_progress_bar


def main(args):
    for passage in get_passages_with_progress_bar(args.passages):
        extracted = constructions.extract_edges(passage, constructions=args.constructions, verbose=args.verbose)
        if any(extracted.values()):
            with tqdm.external_write_mode():
                if not args.verbose:
                    print("%s:" % passage.ID)
                for construction, edges in extracted.items():
                    if edges:
                        print("  %s:" % construction.description)
                        for edge in edges:
                            print("    %s [%s %s]" % (edge, edge.tag, edge.child))
                print()


if __name__ == "__main__":
    argparser = ArgumentParser(description="Extract linguistic constructions from UCCA corpus.")
    argparser.add_argument("passages", nargs="+", help="the corpus, given as xml/pickle file names")
    constructions.add_argument(argparser, False)
    argparser.add_argument("-v", "--verbose", action="store_true", help="print tagged text for each passage")
    main(argparser.parse_args())



import pytest

from ucca import layer0, convert, textutil
from .conftest import crossing, multi_sent, l1_passage, discontiguous, empty, PASSAGES

"""Tests the textutil module functions and classes."""


@pytest.mark.parametrize("create, breaks", (
        (multi_sent, [4, 7, 11]),
        (crossing, [3, 7]),
        (discontiguous, [20]),
        (l1_passage, [20]),
        (empty, []),
))
def test_break2sentences(create, breaks):
    """Tests identifying correctly sentence ends. """
    assert textutil.break2sentences(create()) == breaks


def test_word_vectors():
    vectors, dim = textutil.get_word_vectors()
    for word, vector in vectors.items():
        assert len(vector) == dim, "Vector dimension for %s is %d != %d" % (word, len(vector), dim)


@pytest.mark.parametrize("create", PASSAGES)
@pytest.mark.parametrize("as_array", (True, False))
def test_annotate_passage(create, as_array):
    passage = create()
    textutil.annotate(passage, as_array=as_array)
    for p in passage, convert.from_standard(convert.to_standard(passage)):
        assert textutil.is_annotated(p, as_array=as_array), "Passage %s is not annotated" % passage.ID
        for terminal in p.layer(layer0.LAYER_ID).all:
            if as_array:
                assert terminal.tok is not None, "Terminal %s has no annotation" % terminal
                assert len(terminal.tok) == len(textutil.Attr)
            else:
                for attr in textutil.Attr:
                    assert attr.key in terminal.extra, "Terminal %s has no %s" % (terminal, attr.name)


@pytest.mark.parametrize("as_array", (True, False))
@pytest.mark.parametrize("convert_and_back", (True, False))
def test_annotate_all(as_array, convert_and_back):
    passages = [create() for create in PASSAGES]
    list(textutil.annotate_all(passages))
    for passage, compare in textutil.annotate_all(((p, p) for p in passages), as_array=as_array, as_tuples=True):
        assert passage is compare
        p = (passage, convert.from_standard(convert.to_standard(passage)))[convert_and_back]
        assert textutil.is_annotated(p, as_array=as_array), "Passage %s is not annotated" % passage.ID
        for terminal in p.layer(layer0.LAYER_ID).all:
            if as_array:
                assert terminal.tok is not None, "Terminal %s in passage %s has no annotation" % (terminal, passage.ID)
                assert len(terminal.tok) == len(textutil.Attr)
            else:
                for attr in textutil.Attr:
                    assert attr.key in terminal.extra, "Terminal %s in passage %s has no %s" % (
                        terminal, passage.ID, attr.name)
